import 'package:dartz/dartz.dart';
import 'package:meta/meta.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/usecases/usecase.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_get_enrollment_form_response.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/repositories/user_repository.dart';
import 'package:equatable/equatable.dart';

class UserGetEnrollmentForm
    implements
        UseCase<UserGetEnrollmentFormResponse, UserGetEnrollmentFormParams> {
  final UserRepository repository;

  UserGetEnrollmentForm(
    this.repository,
  );

  @override
  Future<Either<Failure, UserGetEnrollmentFormResponse>> call(
      UserGetEnrollmentFormParams params) async {
    return await repository.getEnrollmentForm(
      jwt: params.jwt,
    );
  }
}

class UserGetEnrollmentFormParams extends Equatable {
  final String jwt;

  UserGetEnrollmentFormParams({
    @required this.jwt,
  });

  @override
  List<Object> get props => [
        jwt,
      ];
}
